# © Copyright, Fervent | All Rights Reserved
"""
# ====================================
# Exploring Stock Price & Returns Data
# ====================================

Are You A Beginner?
We STRONGLY recommend using the .ipynb version instead of this .py version
The .ipynb has more explanatory notes to help and guide you through.

The .py version is largely designed for more intermediate / advanced users of
Python.
"""

# Import package dependencies
import pandas as pd  # for general data wrangling / exploration
import matplotlib.pyplot as plt  # for plotting
import seaborn as sns  # for making plots look good
sns.set()  # implementing seaborn chart templates / style

# Load the stock price data
df = pd.read_csv('../data/stock_prices.csv')

# Get total number of firms in the dataset
print(df['ticker'].nunique())

# Note that the date is stored as a 'string' format.
print(df['date'][0])

# Verifying the type and seeing that it's a string (str)
print(type(df['date'][0]))

# Convert the date to a timestamp object
df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')

# Note that the date is now stored as a timestamp
print(df['date'][0])

# Verifying the type and seeing that it's a Pandas Timestamp
print(type(df['date'][0]))

# Explore the data (view the sample timeframe)
df['date'].describe()

# Calculating returns (incorrectly)
df['returns_incorrect'] = df['price'].pct_change(1)

# Notice that this results in using the prices of 2 different stocks to calculate returns!
print(df.iloc[3230:3250])

# Calculating returns correctly
df['returns'] = df.groupby('ticker')['price'].pct_change(1)

# Delete the "returns_incorrect" column
df.drop(columns=['returns_incorrect'], inplace=True)

# Explore the data
df.groupby('ticker')['returns'].describe()
df.groupby('ticker')['returns'].max().max()

# Exploring randomness and levels of randomness
df[df['ticker'] == 'CNP']['returns'].plot(figsize=(12, 8))
df[df['ticker'] == 'AKAM']['returns'].plot(figsize=(12, 8))

# BONUS: Better Plots
# Set the date column as the index so the dates show up on the x axis easily
df.set_index('date', inplace=True)

df[df['ticker'] == 'CNP']['returns'].plot(figsize=(12, 8),
                                          color='#39b8eb',  # Fervent Blue
                                          title="Returns of CNP")

df[df['ticker'] == 'AKAM']['returns'].plot(figsize=(12, 8),
                                           color='#39b8eb',  # Fervent Blue
                                           title="Returns of AKAM")
